@import url("champi.css");

div.sphinxsidebarwrapper {
  background-image: url("oomadness_barre_blanc.png");
  background-position: left 5px;
  background-repeat: no-repeat;
}
div.sphinxsidebarwrapper>h3 a {
  padding-top: 90px;
  background-image: url("mini_oomadness.png");
  background-position: left top;
  background-repeat: no-repeat;
  min-width: 210px;
  display: block;
}
