@import url("ec_base.css");
body {
  background-color: #6573b2;
  background-image: url("balazar_bar1.jpeg");
  background-position: right top;
  background-repeat: no-repeat;
  background-attachment: fixed;
  background-size: auto 100%;
}
div.bodywrapper {
  margin: 0 270px 0 270px;
}
div.body {
  margin-bottom: 0px;
  padding-top: 0px;
  padding-left: 0px;
  padding-bottom: 0px;
}
div.footer {
  margin: 20px 250px 10px 250px;
  color: #AAB7E4;
}
div.footer a {
  color: #FFF;
}

div.links { position: fixed; top: 0px; padding: 7px; }

div.related1 {
  color: #AAB7E4;
  padding-top: 5px;
  margin-left: 270px;
  margin-right: 270px;
  width: auto;
  text-align: center;
}
div.related1 a { text-decoration: none; color: white; }
div.related1 a:hover{ text-decoration: underline; }
.smallify {
  color: #BFB;
}
div.related1 .smallify a {
  color: #BFB;
}
div.related2 {
  font-size: 100%;
  margin-right: 230px;
  width: auto;
}
div.related2 a {
  color: #FFF;
}

div.bar1 {
  width: 100%;
  height: 100%;
  position: fixed;
  z-index: -1;
  background-image: url("balazar_bar2.jpeg");
  background-position: left top;
  background-repeat: no-repeat;
  background-size: auto 100%;
}

div.sphinxsidebar {
  float: none;
  width: auto;
  font-size: 100%;
  margin-left: 0px;
  padding-bottom: 30px;

  margin-left: 270px;
  margin-right: 270px;
  text-align: center;
}
div.sphinxsidebarwrapper {
  padding-top: 0px;
  font-size: 120%;
}
div.sphinxsidebarwrapper h3 {
  padding-top: 20px;
  color: #FFF;
  display: none;
}
div.sphinxsidebarwrapper div {
  display: inline;
}
div.sphinxsidebarwrapper div a {
  color: #AAB7E4;
}
div.sphinxsidebarwrapper h3:first-child {
  margin-top: 0px;
  padding-top: 0px;
  display: block;
}
div.sphinxsidebarwrapper>h3 a {
  display: block;
  width: 100%;
  height: 211px;
  background-image: url("balazar_titre.png");
  background-position: center top;
  background-repeat: no-repeat;
  margin-bottom: 1em;
}




div.sphinxsidebar ul, div.sphinxsidebar ul ul, div.sphinxsidebar ul.want-points {
  margin: 0px;
  padding: 0;
  display: inline;
}
div.sphinxsidebar li {
  display: inline;
}
div.sphinxsidebar li a {
  display: none;
}
div.sphinxsidebar li.toctree-l3 a {
  display: inline;
  padding-left: 5px;
  padding-right: 5px;
}
div.sphinxsidebar a { text-decoration: none; color: #FFF; }
div.sphinxsidebar a:hover{ text-decoration: underline; }

h1 a { text-decoration: none; }
h2 a { text-decoration: none; }

a { text-decoration: underline; color: #55F; }
a:hover{ color: #00F; }

h1, h2 {
  display: inline-block;
  background-color: #6573B2;
  padding-top: 5px;
  padding-bottom: 5px;
  padding-left: 10px;
  padding-right: 20px;
  margin-left: -10px;
  margin-right: 0px;
  margin-top: 0px;
  margin-bottom: 0px;
  color: white;
  border-top-left-radius: 10px;
  border-bottom-right-radius: 30px;
}
h1 a, h2 a { text-decoration: none; color: white; }
h1 a:hover, h2 a:hover { text-decoration: underline; color: white; }

.post-box {
  border-right: 1px solid rgba(161, 173, 228, 1);
  border-bottom: 4px solid rgba(161, 173, 228, 1);
  background-color: white;
  border-radius: 15px;
  margin-left: -20px;
  margin-right: -20px;
  margin-top: 10px;
  margin-bottom: 30px;
  padding-left: 20px;
  padding-right: 20px;
  padding-bottom: 20px;
  padding-top: 10px;
}
.body>.section:first-of-type {
  border-right: 1px solid rgba(161, 173, 228, 1);
  border-bottom: 4px solid rgba(161, 173, 228, 1);
  background-color: white;
  border-radius: 15px;
  margin-left: -20px;
  margin-right: -20px;
  margin-top: 10px;
  margin-bottom: 30px;
  padding-left: 20px;
  padding-right: 20px;
  padding-top: 1px;
}
/*.body>.section:first-of-type>p:first-of-type {
  padding-top: 20px;
}
h1+p, h2+p, .post-date+p, h1+.figure, h2+.figure, .post-date+.figure {
    margin-top: 30px;
}
*/
.section {
  margin-bottom: 35px;
}

.in-category {
  display: none;
}
.in-category a { color: #AAA; text-decoration: none; }
.in-category a:hover { text-decoration: underline; }

h3 { color: rgb(113, 209, 129); }

h3, div.body p, div.body dd {
  text-align: justify;
  max-width: 40em;
}
div.body li {
  text-align: justify;
  max-width: 37em;
}

.highlights {
  background-color: #E5E4D8;
  border-radius: 15px;
  padding: 10px;
  margin-left: -10px;
  margin-right: -10px;
  max-width: 40em;
  text-align: justify;
}

.comments {
  border-top: 2px dashed rgba(161, 173, 228, 1);
  padding-top: 15px;
  padding-bottom: 5px;
  margin-left: 2em;
  max-width: 38em;
}

table.docutils {
  margin-bottom: 15px;
}
