@import url("ec_base.css");
body {
  background-color: rgb(229, 239, 224);
  background-image: url("champi_fond.png");
  background-repeat: no-repeat;
  background-size: 100% 190px;
  background-attachment: fixed;
}
div.bodywrapper {
  margin: 0 40px 0 280px;
}
div.body {
  margin-top: 4em;
  padding-bottom: 0px;
}
div.footer {
  margin: 20px 20px 10px 20px;
  color: #6D8F79;
}

div.links {
  position: absolute; padding: 7px;
  background-image: url("champi.svg");
  background-repeat: repeat-x;
  background-size: auto 30px;
  border-top-right-radius: 25px;
  border-top: 1px solid #FFF;
  width: 230px;
  padding-top: 38px;
}

div.related1 {
  font-size: 30pt;
  padding-bottom: 30px;
  margin-top: 10px;
  margin-left: 320px;
  margin-right: 80px;
  font-weight: bold;
  text-align: center;
  width: auto;
  text-shadow: 1px 2px 3px #007;
}
div.related1wrapper {
  margin-left: -20px;
  margin-right: -20px;
  color: white;
}
div.related1 a { text-decoration: none; color: white; }
div.related1 a:hover{ text-decoration: underline; }

div.sphinxsidebar {
  font-size: 100%;
  margin-left: 0px;
  background-color: white;
  border-right: 1px solid rgb(173, 209, 186);
  border-bottom: 4px solid rgb(173, 209, 186);
  border-top-right-radius: 20px;
  border-bottom-right-radius: 15px;
  padding-bottom: 30px;
  padding-top: 35px;
}
div.sphinxsidebarwrapper h3 {
  padding-top: 15px;
  color: #6D8F79;
}
div.sphinxsidebarwrapper>h3 a {
  padding-top: 20px;
  min-width: 210px;
  display: block;
}
div.sphinxsidebar ul {
  margin: 10px;
  padding: 0;
}
div.sphinxsidebar a { text-decoration: none; color: #6D8F79; }
div.sphinxsidebar a:hover{ text-decoration: underline; }

a { text-decoration: underline; color: #6D8F79; }
a:hover{ color: #6D8F79; }

h1, h2 {
  background-image: url("champi.svg");
  background-repeat: repeat-x;
  background-size: auto 100%;
  padding-top: 10px;
  padding-bottom: 10px;
  padding-left: 20px;
  padding-right: 6em; /*6em for date*/
  margin-left: -30px;
  margin-right: -30px;
  margin-top: 8px;
  color: white;
  border-top: 1px solid #FFF;
  border-top-left-radius: 25px;
  border-top-right-radius: 25px;
  text-shadow: 0px 1px 4px #520;
}
h1 {
  margin-top: -5px;
}
h2 a { text-decoration: none; color: white; }
h2 a:hover{ text-decoration: underline; color: white; }

.post-box {
  border-right: 1px solid rgb(173, 209, 186);
  border-bottom: 4px solid rgb(173, 209, 186);
  background-color: white;
  border-radius: 15px;
  margin-left: -20px;
  margin-right: -20px;
  margin-top: 10px;
  margin-bottom: 30px;
  padding-left: 20px;
  padding-right: 20px;
  padding-bottom: 20px;
}
.body>.section:first-of-type {
  border-right: 1px solid rgb(173, 209, 186);
  border-bottom: 4px solid rgb(173, 209, 186);
  background-color: white;
  border-radius: 15px;
  margin-left: -20px;
  margin-right: -20px;
  margin-top: 10px;
  margin-bottom: 30px;
  padding-left: 20px;
  padding-right: 20px;
  padding-top: 1px;
}
.in-category {
  color: #AAA;
  text-align: right;
  margin-left: 1em;
  margin-top: -1em;
  margin-bottom: -0.5em;
}
.in-category a { color: #AAA; text-decoration: none; }
.in-category a:hover { text-decoration: underline; }


div.sphinxsidebar input {
  border: 1px solid #6D8F79;
}

h3, div.body p, div.body dd {
  text-align: justify;
  max-width: 40em;
}
div.body li {
  text-align: justify;
  max-width: 37em;
}

.highlights {
  border: 1px solid #E5C1C1;
  border-bottom: 4px solid rgb(173, 209, 186);
  background-color: white;
  border-radius: 15px;
  padding: 10px;
  margin-left: -10px;
  margin-right: -10px;
  max-width: 40em;
  text-align: justify;
}

.comments {
  border-top: 2px dashed rgb(173, 209, 186);
  padding-top: 15px;
  padding-bottom: 5px;
  margin-left: 2em;
  max-width: 38em;
}

table.docutils {
  margin-bottom: 15px;
}

.post-date {
  float: right;
  margin-top: -3.2em;
  margin-left: 2em;
  color: white;
}
