@import url("basic.css");
body {
  margin: 0;
  padding: 0;
  font-family: sans;
}

a em { font-style:normal; }

a { text-decoration: underline; color: #3B4; }
a:hover{ color: #5C6; }

div.body {
  min-width: 450px;
  max-width: 830px;
}

div.admonition p.admonition-title + p {
    display: inline;
}
div.admonition p {
    margin-bottom: 5px;
}
div.admonition pre {
    margin-bottom: 5px;
}
div.admonition ul, div.admonition ol {
    margin-bottom: 5px;
}
div.note {
    background-color: #eee;
    border: 1px solid #ccc;
}
div.seealso {
    background-color: #ffc;
    border: 1px solid #ff6;
}
div.topic {
    background-color: #eee;
}
div.warning {
    background-color: #ffe4e4;
    border: 1px solid #f66;
}
p.admonition-title {
    display: inline;
}

pre {
    padding: 5px;
    line-height: 120%;
    border: 1px solid #ac9;
    border-left: none;
    border-right: none;
    
}
tt {
    background-color: #ecf0f3;
    padding: 0 1px 0 1px;
    font-size: 0.95em;
}
th {
    background-color: #ede;
}
.warning tt {
    background: #efc2c2;
}
.note tt {
    background: #d6d6d6;
}
div.viewcode-block:target {
    background-color: #f4debf;
    border-top: 1px solid #ac9;
    border-bottom: 1px solid #ac9;
}

div.sphinxsidebar {
  float: left;
  width: 230px;
}
div.sphinxsidebarwrapper {
  padding: 10px 5px 0 10px;
}
div.sphinxsidebar input {
  width: 170px;
}
div.sphinxsidebar ul {
  list-style: none;
}
li.current a {
  font-weight: bold;
}
div.sphinxsidebar ul ul {
  margin-left: 20px;
  margin-top: 0;
  margin-bottom: 0;
}

div.sphinxsidebar ul ul, div.sphinxsidebar ul.want-points {
  list-style: none outside none;
}

.nav_bar {
  float: right;
  padding-right: 1.5em;
  padding-bottom: 1.5em;
}

.float-right { float: right; padding-top: 15px; padding-left: 20px; clear: right; }
.gallery { margin-top: 15px; margin-bottom: 10px; }
.float-right .gallery { width: 210px; }
.float-right .gallery .gallery-img { padding-bottom: 5px; }

.video {
  background-repeat: no-repeat;
  background-size: 100% 100%;
  width: 640px;
  height: 480px;
  text-align: center;
}
.video-read {
  font-size: 400%;
  font-weight: bold;
  color: white;
  background-color: rgba(0, 0, 0, 0.65);
  border-radius: 15px;
  text-align: center;
  width: 1.5em;
  display: inline-block;
  position: relative;
  top: 200px;
}
.video-read:hover {
  background-color: red;
}

.post-date {
  float: right;
  margin-left: 1em;
  margin-bottom: 1em;
}
.post-box {
  clear: right;
}

a.current { font-weight: bold; }

.link-lang img {
  width: 2em;
  height: 1.4em;
  vertical-align: middle;
}
.link-rss img {
  width: 1.4em;
  height: 1.4em;
  vertical-align: middle;
}
.localtoc { display: none; }

.section li p {
  margin-top: 2px;
  margin-bottom: 3px;
}

table {
  border-collapse: collapse;
}
table, th, td {
  border: 1px solid #aaa;
} 
td {
  padding: 4px;
} 
tr:nth-child(even){
  background-color: #f2f2f2;
}

.literal-block {
  background: #eeffcc;
}
.keyword {
  color: #007020;
  font-weight: bold;
}
.string {
  color: red;
}
.literal-block .comment {
  color: gray;
}

.highlighted {
  background-color: yellow;
}

.search_results {
  padding-top: 1em;
  padding-bottom: 0.8em;
}
.search_result {
  padding-bottom: 0.8em;
}
.search_excerpt {
  margin-top: 0.3em;
  margin-left: 2em;
  color: gray;
}

.bottom {
  clear: both;
  margin-left: 1em;
  padding-top: 1em;
  margin-bottom: 0.5em;
}
